@@ -1,8 +1,8 @@
-from google.cloud.bigquery import RangePartitioning, PartitionRange, TimePartitioning
+from google.cloud.bigquery import PartitionRange, RangePartitioning, TimePartitioning
+from google.cloud.bigquery.job import LoadJobConfig, QueryJobConfig
+from google.cloud.exceptions import NotFound
 
 from skt.ye import get_spark
-from google.cloud.bigquery.job import QueryJobConfig, LoadJobConfig
-from google.cloud.exceptions import NotFound
 
 PROJECT_ID = "sktaic-datahub"
 TEMP_DATASET = "temp_1d"
@@ -11,7 +11,9 @@ CREDENTIALS_SECRET_PATH = "gcp/sktaic-datahub/dataflow"
 
 def get_credentials():
     import json
+
     from google.oauth2 import service_account
+
     from skt.vault_utils import get_secrets
 
     key = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
@@ -24,9 +26,10 @@ def get_credentials():
 
 def set_default_credentials():
     import os
-    from skt.vault_utils import get_secrets
     import uuid
 
+    from skt.vault_utils import get_secrets
+
     key_json = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
     key_json_path = f"/tmp/{str(uuid.uuid4()).replace('-', '_')}"
     with open(key_json_path, "w") as f:
@@ -43,11 +46,11 @@ def get_bigquery_storage_client(credentials=None):
     return bigquery_storage.BigQueryReadClient(credentials=credentials)
 
 
-def _get_result_schema(sql, bq_client=None):
+def _get_result_schema(sql, bq_client=None, project_id=PROJECT_ID):
     from google.cloud.bigquery.job import QueryJobConfig
 
     if bq_client is None:
-        bq_client = get_bigquery_client()
+        bq_client = get_bigquery_client(project_id=project_id)
     job_config = QueryJobConfig(
         dry_run=True,
         use_query_cache=False,
@@ -57,7 +60,7 @@ def _get_result_schema(sql, bq_client=None):
     return schema
 
 
-def _get_result_column_type(sql, column, bq_client=None):
+def _get_result_column_type(sql, column, bq_client=None, project_id=PROJECT_ID):
     schema = _get_result_schema(sql, bq_client)
     fields = schema["fields"]
     r = [field["type"] for field in fields if field["name"] == column]
@@ -82,11 +85,11 @@ def _print_query_job_results(query_job):
         print(e)
 
 
-def bq_insert_overwrite_table(sql, destination):
-    bq = get_bigquery_client()
+def bq_insert_overwrite_table(sql, destination, project_id=PROJECT_ID):
+    bq = get_bigquery_client(project_id=project_id)
     table = bq.get_table(destination)
     if table.time_partitioning or table.range_partitioning:
-        load_query_result_to_partitions(sql, destination)
+        load_query_result_to_partitions(sql, destination, project_id)
     else:
         config = QueryJobConfig(
             destination=destination,
@@ -100,13 +103,13 @@ def bq_insert_overwrite_table(sql, destination):
         bq.close()
 
 
-def bq_ctas(sql, destination=None, partition_by=None, clustering_fields=None):
+def bq_ctas(sql, destination=None, partition_by=None, clustering_fields=None, project_id=PROJECT_ID):
     """
     create new table and insert results
     """
     from google.cloud.bigquery.job import QueryJobConfig
 
-    bq = get_bigquery_client()
+    bq = get_bigquery_client(project_id=project_id)
     if partition_by:
         partition_type = _get_result_column_type(sql, partition_by, bq_client=bq)
         if partition_type == "DATE":
@@ -145,14 +148,14 @@ def bq_ctas(sql, destination=None, partition_by=None, clustering_fields=None):
     return job.destination
 
 
-def _bq_query_to_new_table(sql, destination=None):
-    return bq_ctas(sql, destination)
+def _bq_query_to_new_table(sql, destination=None, project_id=PROJECT_ID):
+    return bq_ctas(sql, destination, project_id=project_id)
 
 
-def _bq_query_to_existing_table(sql, destination):
+def _bq_query_to_existing_table(sql, destination, project_id=PROJECT_ID):
     from google.cloud.bigquery.job import QueryJobConfig
 
-    bq = get_bigquery_client()
+    bq = get_bigquery_client(project_id=project_id)
     table = bq.get_table(destination)
     config = QueryJobConfig(
         destination=destination,
@@ -169,9 +172,9 @@ def _bq_query_to_existing_table(sql, destination):
     return job.destination
 
 
-def _bq_table_to_pandas(table):
+def _bq_table_to_pandas(table, project_id=PROJECT_ID):
     credentials = get_credentials()
-    bq = get_bigquery_client(credentials=credentials)
+    bq = get_bigquery_client(credentials=credentials, project_id=project_id)
     bqstorage_client = get_bigquery_storage_client(credentials=credentials)
     row_iterator = bq.list_rows(table)
     df = row_iterator.to_dataframe(bqstorage_client=bqstorage_client, progress_bar_type="tqdm")
@@ -181,18 +184,20 @@ def _bq_table_to_pandas(table):
 
 
 def _bq_cell_magic(line, query):
-    from IPython.core import magic_arguments
-    from google.cloud.bigquery.magics.magics import _cell_magic
     import time
 
+    from google.cloud.bigquery.magics.magics import _cell_magic
+    from IPython.core import magic_arguments
+
     start = time.time()
     args = magic_arguments.parse_argstring(_cell_magic, line)
 
     if args.params is not None:
         try:
-            from google.cloud.bigquery.dbapi import _helpers
             import ast
 
+            from google.cloud.bigquery.dbapi import _helpers
+
             params = _helpers.to_query_parameters(ast.literal_eval("".join(args.params)), {})
             query_params = dict()
             for p in params:
@@ -227,6 +232,7 @@ def _is_ipython():
 def set_gcp_credentials():
     import os
     import tempfile
+
     from skt.vault_utils import get_secrets
 
     key = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
@@ -260,23 +266,24 @@ def get_bigquery_client(credentials=None, project_id=PROJECT_ID):
     return bigquery.Client(credentials=credentials, project=project_id)
 
 
-def bq_table_exists(table):
+def bq_table_exists(table, project_id=PROJECT_ID):
     try:
-        get_bigquery_client().get_table(table)
+        get_bigquery_client(project_id=project_id).get_table(table)
     except NotFound:
         return False
     return True
 
 
-def get_unhidden_partitions(table_name):
+def get_unhidden_partitions(table_name, project_id=PROJECT_ID):
     parts = filter(
-        lambda x: x not in ["__NULL__", "__UNPARTITIONED__"], get_bigquery_client().list_partitions(table_name)
+        lambda x: x not in ["__NULL__", "__UNPARTITIONED__"],
+        get_bigquery_client(project_id=project_id).list_partitions(table_name),
     )
     return parts
 
 
-def _get_partition_filter(dataset, table_name, partition):
-    table = get_bigquery_client().get_table(f"{dataset}.{table_name}")
+def _get_partition_filter(dataset, table_name, partition, project_id=PROJECT_ID):
+    table = get_bigquery_client(project_id=project_id).get_table(f"{dataset}.{table_name}")
     if "timePartitioning" in table._properties:
         partition_column_name = table._properties["timePartitioning"]["field"]
         return f"{partition_column_name} = '{partition}'"
@@ -286,16 +293,19 @@ def _get_partition_filter(dataset, table_name, partition):
     return ""
 
 
-def bq_to_pandas(sql, large=False):
+def bq_to_pandas(sql, large=False, project_id=PROJECT_ID):
     destination = None
     if large:
-        destination = get_temp_table()
-    destination = _bq_query_to_new_table(sql, destination)
-    return _bq_table_to_pandas(destination)
+        destination = get_temp_table(project_id=project_id)
+    destination = _bq_query_to_new_table(sql, destination, project_id=project_id)
+    return _bq_table_to_pandas(destination, project_id=project_id)
 
 
-def _bq_table_to_df(dataset, table_name, col_list, partition=None, where=None, spark_session=None):
+def _bq_table_to_df(
+    dataset, table_name, col_list, partition=None, where=None, spark_session=None, project_id=PROJECT_ID
+):
     import base64
+
     from skt.vault_utils import get_secrets
 
     if not spark_session:
@@ -304,12 +314,12 @@ def _bq_table_to_df(dataset, table_name, col_list, partition=None, where=None, s
     key = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
     df = (
         spark_session.read.format("bigquery")
-        .option("project", "sktaic-datahub")
-        .option("table", f"sktaic-datahub:{dataset}.{table_name}")
+        .option("project", project_id)
+        .option("table", f"{project_id}:{dataset}.{table_name}")
         .option("credentials", base64.b64encode(key.encode()).decode())
     )
     if partition:
-        table = get_bigquery_client().get_table(f"{dataset}.{table_name}")
+        table = get_bigquery_client(project_id=project_id).get_table(f"{dataset}.{table_name}")
         if "timePartitioning" in table._properties:
             partition_column_name = table._properties["timePartitioning"]["field"]
             filter = f"{partition_column_name} = '{partition}'"
@@ -326,39 +336,53 @@ def _bq_table_to_df(dataset, table_name, col_list, partition=None, where=None, s
     return df
 
 
-def bq_table_to_df(dataset, table_name, col_list, partition=None, where=None, spark_session=None):
-    return _bq_table_to_df(dataset, table_name, col_list, partition, where, spark_session)
+def bq_table_to_df(
+    dataset, table_name, col_list, partition=None, where=None, spark_session=None, project_id=PROJECT_ID
+):
+    return _bq_table_to_df(dataset, table_name, col_list, partition, where, spark_session, project_id=project_id)
 
 
-def bq_table_to_parquet(dataset, table_name, output_dir, col_list="*", partition=None, where=None, mode="overwrite"):
-    bq_table_to_df(dataset, table_name, col_list, partition=partition, where=where).write.mode(mode).parquet(output_dir)
+def bq_table_to_parquet(
+    dataset, table_name, output_dir, col_list="*", partition=None, where=None, mode="overwrite", project_id=PROJECT_ID
+):
+    bq_table_to_df(dataset, table_name, col_list, partition=partition, where=where, project_id=project_id).write.mode(
+        mode
+    ).parquet(output_dir)
 
 
-def bq_table_to_pandas(dataset, table_name, col_list="*", partition=None, where=None):
+def bq_table_to_pandas(dataset, table_name, col_list="*", partition=None, where=None, project_id=PROJECT_ID):
     col_list_str = ", ".join(col_list) if isinstance(col_list, list) else col_list
     query = f"SELECT {col_list_str} FROM {dataset}.{table_name} "
     if partition or where:
         query += "WHERE "
         conditions = []
         if partition:
-            conditions.append(f" {_get_partition_filter(dataset, table_name, partition)} ")
+            conditions.append(f" {_get_partition_filter(dataset, table_name, partition, project_id=project_id)} ")
         if where:
             conditions.append(f" {where} ")
         query += " AND ".join(conditions)
-    return bq_to_pandas(query=query)
+    return bq_to_pandas(query=query, project_id=project_id)
 
 
 def _df_to_bq_table(
-    df, dataset, table_name, partition=None, partition_field=None, clustering_fields=None, mode="overwrite"
+    df,
+    dataset,
+    table_name,
+    partition=None,
+    partition_field=None,
+    clustering_fields=None,
+    mode="overwrite",
+    project_id=PROJECT_ID,
 ):
     import base64
+
     from skt.vault_utils import get_secrets
 
     key = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
     table = f"{dataset}.{table_name}${partition}" if partition else f"{dataset}.{table_name}"
     df = (
         df.write.format("bigquery")
-        .option("project", "sktaic-datahub")
+        .option("project", project_id)
         .option("credentials", base64.b64encode(key.encode()).decode())
         .option("table", table)
         .option("temporaryGcsBucket", "temp-seoul-7d")
@@ -371,46 +395,72 @@ def _df_to_bq_table(
 
 
 def df_to_bq_table(
-    df, dataset, table_name, partition=None, partition_field=None, clustering_fields=None, mode="overwrite"
+    df,
+    dataset,
+    table_name,
+    partition=None,
+    partition_field=None,
+    clustering_fields=None,
+    mode="overwrite",
+    project_id=PROJECT_ID,
 ):
-    _df_to_bq_table(df, dataset, table_name, partition, partition_field, clustering_fields, mode)
+    _df_to_bq_table(df, dataset, table_name, partition, partition_field, clustering_fields, mode, project_id=project_id)
 
 
 def parquet_to_bq_table(
-    parquet_dir, dataset, table_name, partition=None, partition_field=None, clustering_fields=None, mode="overwrite"
+    parquet_dir,
+    dataset,
+    table_name,
+    partition=None,
+    partition_field=None,
+    clustering_fields=None,
+    mode="overwrite",
+    project_id=PROJECT_ID,
 ):
     try:
         spark = get_spark()
         df = spark.read.format("parquet").load(parquet_dir)
-        _df_to_bq_table(df, dataset, table_name, partition, partition_field, clustering_fields, mode)
+        _df_to_bq_table(
+            df, dataset, table_name, partition, partition_field, clustering_fields, mode, project_id=project_id
+        )
     finally:
         spark.stop()
 
 
 def pandas_to_bq_table(
-    pd_df, dataset, table_name, partition=None, partition_field=None, clustering_fields=None, mode="overwrite"
+    pd_df,
+    dataset,
+    table_name,
+    partition=None,
+    partition_field=None,
+    clustering_fields=None,
+    mode="overwrite",
+    project_id=PROJECT_ID,
 ):
     try:
         spark = get_spark()
         spark_df = spark.createDataFrame(pd_df)
-        _df_to_bq_table(spark_df, dataset, table_name, partition, partition_field, clustering_fields, mode)
+        _df_to_bq_table(
+            spark_df, dataset, table_name, partition, partition_field, clustering_fields, mode, project_id=project_id
+        )
     finally:
         spark.stop()
 
 
-def pandas_to_bq(pd_df, destination, partition=None, clustering_fields=None, overwrite=True):
+def pandas_to_bq(pd_df, destination, partition=None, clustering_fields=None, overwrite=True, project_id=PROJECT_ID):
     range_partitioning = None
     time_partitioning = None
-    bq = get_bigquery_client()
-    if bq_table_exists(destination):
+    bq = get_bigquery_client(project_id=project_id)
+    if bq_table_exists(destination, project_id=project_id):
         target_table = bq.get_table(destination)
         range_partitioning = target_table.range_partitioning
         time_partitioning = target_table.time_partitioning
     else:
         if partition:
-            from pandas.api.types import is_integer_dtype
             import datetime
 
+            from pandas.api.types import is_integer_dtype
+
             if is_integer_dtype(pd_df[partition][0]):
                 range_partitioning = RangePartitioning(
                     PartitionRange(start=200001, end=209912, interval=1), field=partition
@@ -463,25 +513,25 @@ def rdd_to_pandas(func):
     return _rdd_to_pandas
 
 
-def bq_to_df(query, spark_session=None):
-    temp_table_name = get_temp_table()
+def bq_to_df(query, spark_session=None, project_id=PROJECT_ID):
+    temp_table_name = get_temp_table(project_id=project_id)
     jc = QueryJobConfig(
         create_disposition="CREATE_IF_NEEDED",
         write_disposition="WRITE_TRUNCATE",
         destination=temp_table_name,
     )
-    bq_client = get_bigquery_client()
+    bq_client = get_bigquery_client(project_id=project_id)
     job = bq_client.query(query, job_config=jc)
     job.result()
     t = temp_table_name.split(".")
 
-    return _bq_table_to_df(t[1], t[2], "*", spark_session=spark_session)
+    return _bq_table_to_df(t[1], t[2], "*", spark_session=spark_session, project_id=project_id)
 
 
-def load_query_result_to_table(dest_table, query, part_col_name=None, clustering_fields=None):
-    bq_client = get_bigquery_client()
+def load_query_result_to_table(dest_table, query, part_col_name=None, clustering_fields=None, project_id=PROJECT_ID):
+    bq_client = get_bigquery_client(project_id=project_id)
     print(query)
-    if bq_table_exists(dest_table):
+    if bq_table_exists(dest_table, project_id=project_id):
         table = bq_client.get_table(dest_table)
         qjc = QueryJobConfig(
             destination=dest_table,
@@ -494,7 +544,7 @@ def load_query_result_to_table(dest_table, query, part_col_name=None, clustering
         job = bq_client.query(query, job_config=qjc)
         job.result()
     else:
-        temp_table_name = get_temp_table()
+        temp_table_name = get_temp_table(project_id=project_id)
         bq_client.query(f"CREATE OR REPLACE TABLE {temp_table_name} AS {query}").result()
         if part_col_name:
             schema = bq_client.get_table(temp_table_name).schema
@@ -529,11 +579,11 @@ def load_query_result_to_table(dest_table, query, part_col_name=None, clustering
         bq_client.query(f"SELECT * FROM {temp_table_name}", job_config=qjc).result()
 
 
-def get_temp_table():
+def get_temp_table(project_id=PROJECT_ID):
     import uuid
 
     table_id = str(uuid.uuid4()).replace("-", "_")
-    full_table_id = f"{PROJECT_ID}.{TEMP_DATASET}.{table_id}"
+    full_table_id = f"{project_id}.{TEMP_DATASET}.{table_id}"
 
     return full_table_id
 
@@ -547,11 +597,11 @@ def _get_partition_name(table):
         raise Exception(f"Table[{table}] is not partitioned.")
 
 
-def load_query_result_to_partitions(query, dest_table):
-    from google.cloud.bigquery.table import TableReference
+def load_query_result_to_partitions(query, dest_table, project_id=PROJECT_ID):
     from google.cloud.bigquery.dataset import DatasetReference
+    from google.cloud.bigquery.table import TableReference
 
-    bq = get_bigquery_client()
+    bq = get_bigquery_client(project_id=project_id)
     table = bq.get_table(dest_table)
 
     """
@@ -571,7 +621,7 @@ def load_query_result_to_partitions(query, dest_table):
         _print_query_job_results(job)
         return dest_table
 
-    temp_table_id = get_temp_table()
+    temp_table_id = get_temp_table(project_id=project_id)
     qjc = QueryJobConfig(
         destination=temp_table_id,
         write_disposition="WRITE_TRUNCATE",
@@ -624,11 +674,11 @@ def load_query_result_to_partitions(query, dest_table):
     return partitions
 
 
-def get_max_part(table_name):
+def get_max_part(table_name, project_id=PROJECT_ID):
     from datetime import datetime
 
-    bq_client = get_bigquery_client()
-    parts = get_unhidden_partitions(table_name)
+    bq_client = get_bigquery_client(project_id=project_id)
+    parts = get_unhidden_partitions(table_name, project_id=project_id)
 
     if not parts:
         raise Exception("Max partition value is invalid or null.")
@@ -644,17 +694,21 @@ def get_max_part(table_name):
         raise Exception("Partition column is neither DATE or INTEGER type.")
 
 
-def bq_insert_overwrite(sql, destination, suffixes=None, partition=None, clustering_fields=None):
+def bq_insert_overwrite(sql, destination, suffixes=None, partition=None, clustering_fields=None, project_id=PROJECT_ID):
     if suffixes:
-        bq_insert_overwrite_with_suffixes(sql, destination, suffixes, partition, clustering_fields)
+        bq_insert_overwrite_with_suffixes(
+            sql, destination, suffixes, partition, clustering_fields, project_id=project_id
+        )
     else:
-        bq_insert_overwrite_without_suffixes(sql, destination, partition, clustering_fields)
+        bq_insert_overwrite_without_suffixes(sql, destination, partition, clustering_fields, project_id=project_id)
 
 
-def bq_insert_overwrite_with_suffixes(sql, destination, suffixes, partition=None, clustering_fields=None):
-    temp_table = get_temp_table()
-    bq_ctas(sql, temp_table, partition_by=partition, clustering_fields=clustering_fields)
-    bq = get_bigquery_client()
+def bq_insert_overwrite_with_suffixes(
+    sql, destination, suffixes, partition=None, clustering_fields=None, project_id=PROJECT_ID
+):
+    temp_table = get_temp_table(project_id=project_id)
+    bq_ctas(sql, temp_table, partition_by=partition, clustering_fields=clustering_fields, project_id=project_id)
+    bq = get_bigquery_client(project_id=project_id)
     r = bq.query(f"""select distinct {', '.join(suffixes)} from {temp_table}""")
     for cols in r:
         suffix = "".join([f"__{col}" for col in cols])
@@ -662,15 +716,19 @@ def bq_insert_overwrite_with_suffixes(sql, destination, suffixes, partition=None
         where_clause = " and ".join([f"{x[0]} = '{x[1]}'" for x in zip(suffixes, cols)])
         target = "".join([destination, suffix])
         sub_sql = f"{select_clause} where {where_clause}"
-        bq_insert_overwrite_without_suffixes(sub_sql, target, partition=partition, clustering_fields=clustering_fields)
+        bq_insert_overwrite_without_suffixes(
+            sub_sql, target, partition=partition, clustering_fields=clustering_fields, project_id=project_id
+        )
     bq.close()
 
 
-def bq_insert_overwrite_without_suffixes(sql, destination, partition=None, clustering_fields=None):
-    if bq_table_exists(destination):
-        bq_insert_overwrite_table(sql, destination)
+def bq_insert_overwrite_without_suffixes(
+    sql, destination, partition=None, clustering_fields=None, project_id=PROJECT_ID
+):
+    if bq_table_exists(destination, project_id=project_id):
+        bq_insert_overwrite_table(sql, destination, project_id=project_id)
     else:
-        bq_ctas(sql, destination, partition_by=partition, clustering_fields=clustering_fields)
+        bq_ctas(sql, destination, partition_by=partition, clustering_fields=clustering_fields, project_id=project_id)
 
 
 def get_storage_client(credentials=None, project_id=PROJECT_ID):
